<iframe src="../.ibox.html?raw=true" style="border:none; position:fixed; width:40px; right:0; z-index=999;"></iframe>

# User Defined Types

Similarly to defining implementations, one can define _types_. A type can be either a 'sum' type
(**or** variants), or a 'product' type (**and**). Sum types can also contain nested fields.
Defining a type uses the `def-ty` command.

## Syntax
---
To define a `Point` type with an `x` and `y`, a **_product_** type is defined: `def-ty Point { x:Num
y:Num }`. Each field must be typed. By defining this type, a value constructor is supplied, which
is just a _command_ called `Point`, taking two parameters, x and y.
```plaintext
>> def-ty Point { x:Num y:Num }
()
>> Point --help
Help: `Point`
--> shell:0
 | initialise a `Point`
 |
 | Usage:
 |  => Point x:Number y:Number 

>> Point 1 2
Point (x = 1, y = 2)
```

To define a **_sum_** type, the name of the type is led separated by a double colon, followed by
the variants separated by pipe characters. Sum types represent algebraic data types, so each variant can be a unit type or have
a product type associated with it.
```plaintext
>> def-ty Weather :: Sunny | Rainy { mm:Num } | Cyclonic { name:Str }
()  
>> Weather::Rainy 100
Rainy (mm = 100)
>> Weather::Sunny
Sunny()
>> Weather::Cyclonic 'Marcus'
Cyclonic (name = "Marcus")
```

## Fetching values
---
To extract a value out of a **product** type, the `get` command is used:
```plaintext
>> Point 1 2 | get y
2
>> Point 1 2 | get x
1   
```

## Tuples
---
Tuples are a special type which can be used to quickly group values into a structure without having
to define a type. The command for building a tuple is `Tuple`. Tuples can have 2 or more fields,
named `t0` to `t(n-1)`. The type _signature_ of a tuple is specialised on the size and types of the
fields. `Tuple 1 2` has the type `U_Num-Num_`, while `Tuple 1 'foo' #t` has type `U_Num-Str-Bool_`.
Tuple fields can be accessed through the `get` command: `Tuple 1 2 | get t0`.


> 🔬 Types, especially **sum** types need more development! Please support further development with
> code contributions or sponsorship!
